| Safe Haskell | None |
|---|---|
| Language | Haskell98 |
Language.Haskell.Liquid.Bare.Check
Synopsis
- checkTargetSpec :: [BareSpec] -> TargetSrc -> SEnv SortedReft -> [CoreBind] -> TargetSpec -> Either Diagnostics ()
- checkBareSpec :: BareSpec -> Either Diagnostics ()
- checkTargetSrc :: Config -> BareSpec -> TargetSrc -> Either Diagnostics ()
- checkStratTys :: BareSpec -> TargetSrc -> Either Diagnostics [Name]
- tyCompat :: Var -> RType RTyCon RTyVar r -> Bool
Documentation
checkTargetSpec :: [BareSpec] -> TargetSrc -> SEnv SortedReft -> [CoreBind] -> TargetSpec -> Either Diagnostics () Source #
Checking TargetSpec
checkBareSpec :: BareSpec -> Either Diagnostics () Source #
Checking BareSpec ------------------------------------------------------------------------
checkTargetSrc :: Config -> BareSpec -> TargetSrc -> Either Diagnostics () Source #
Checking TargetSrc ------------------------------------------------------------------------
checkStratTys :: BareSpec -> TargetSrc -> Either Diagnostics [Name] Source #
Check that all stratified types have their constructors defined with refinement type signatures in the BareSpec.
Yields the names of the data constructors of the stratified types.